@@ -1,6 +1,7 @@
 """
-This file demonstrates writing tests using the unittest module. These will pass
-when you run "manage.py test".
+Project: Farnsworth
+
+Authors: Karandeep Singh Nagra and Nader Morshed
 """
 
 from datetime import date, timedelta
@@ -89,6 +90,12 @@ class TestLogin(TestCase):
     def test_homepage(self):
         response = self.client.get(reverse("homepage"), follow=True)
         self.assertRedirects(response, reverse('external'))
+        self.assertContains(response, "Login")
+        self.assertContains(response, "Request Account")
+        self.assertContains(response, "Source")
+        self.assertContains(response, "Help")
+        self.assertContains(response, "Site Map")
+
         self.client.login(username="u", password="pwd")
         response = self.client.get(reverse("homepage"))
         self.assertEqual(response.status_code, 200)
@@ -170,7 +177,7 @@ class TestHomepage(TestCase):
         thread = Thread.objects.get(subject="Thread Subject Test")
         self.assertEqual(thread.owner, self.profile)
 
-    def test_announcment_post(self):
+    def test_announcement_post(self):
         url = reverse("homepage")
         response = self.client.post(url, {
             "announce-manager": self.manager.pk,
@@ -201,10 +208,10 @@ class TestHomepage(TestCase):
         url = reverse("homepage")
         response = self.client.post(url, {
             "add_response-{0}".format(self.req.pk): "",
-            "response-body": "You betcha",
-            "response-action": Response.NONE,
+            "{0}-response-body".format(self.req.pk): "You betcha",
+            "{0}-response-action".format(self.req.pk): Response.NONE,
             }, follow=True)
-        self.assertRedirects(response, url)
+        self.assertEquals(response.status_code, 200)
         self.assertContains(response, "You betcha")
         self.assertNotContains(response, MESSAGES['REQ_CLOSED'])
         self.assertNotContains(response, MESSAGES['REQ_FILLED'])
@@ -213,10 +220,10 @@ class TestHomepage(TestCase):
         url = reverse("homepage")
         response = self.client.post(url, {
             "add_response-{0}".format(self.req.pk): "",
-            "response-body": "You betcha",
-            "response-action": Response.CLOSED,
+            "{0}-response-body".format(self.req.pk): "You betcha",
+            "{0}-response-action".format(self.req.pk): Response.CLOSED,
             }, follow=True)
-        self.assertRedirects(response, url)
+        self.assertEquals(response.status_code, 200)
         # We shouldn't see the request body on the homepage any more when it is
         # filled
         self.assertNotContains(response, "You betcha")
@@ -227,10 +234,10 @@ class TestHomepage(TestCase):
         url = reverse("homepage")
         response = self.client.post(url, {
             "add_response-{0}".format(self.req.pk): "",
-            "response-body": "You betcha",
-            "response-action": Response.FILLED,
+            "{0}-response-body".format(self.req.pk): "You betcha",
+            "{0}-response-action".format(self.req.pk): Response.FILLED,
             }, follow=True)
-        self.assertRedirects(response, url)
+        self.assertEquals(response.status_code, 200)
         # We shouldn't see the request body on the homepage any more when it is
         # filled
         self.assertNotContains(response, "You betcha")
@@ -247,10 +254,7 @@ class TestHomepage(TestCase):
             "pin-pinned": False,
             }, follow=True)
         self.assertRedirects(response, url)
-        self.assertEqual(
-            False,
-            Announcement.objects.get(pk=self.announce.pk).pinned,
-            )
+        self.assertFalse(Announcement.objects.get(pk=self.announce.pk).pinned)
 
     def test_upvote(self):
         # Upvote
@@ -730,8 +734,8 @@ class TestProfilePages(TestCase):
         self.assertContains(response, "Your password was successfully changed.")
 
         self.client.logout()
-        self.assertEqual(False, self.client.login(username="u", password="pwd"))
-        self.assertEqual(True, self.client.login(username="u", password="Jenkins"))
+        self.assertFalse(self.client.login(username="u", password="pwd"))
+        self.assertTrue(self.client.login(username="u", password="Jenkins"))
 
     def test_confirm_password(self):
         url = reverse("my_profile")
@@ -749,8 +753,8 @@ class TestProfilePages(TestCase):
             response, "The two password fields didn't match.".replace("'", "&#39;")
             )
         self.client.logout()
-        self.assertEqual(False, self.client.login(username="u", password="Jenkins"))
-        self.assertEqual(True, self.client.login(username="u", password="pwd"))
+        self.assertFalse(self.client.login(username="u", password="Jenkins"))
+        self.assertTrue(self.client.login(username="u", password="pwd"))
 
     def test_visible(self):
         self.oprofile.email_visible = True
@@ -881,8 +885,8 @@ class TestModifyUser(TestCase):
         self.assertContains(response,
                             MESSAGES['USER_PW_CHANGED'].format(username=self.u.username))
         self.client.logout()
-        self.assertEqual(False, self.client.login(username="u", password="pwd"))
-        self.assertEqual(True, self.client.login(username="u", password="Leeroy"))
+        self.assertFalse(self.client.login(username="u", password="pwd"))
+        self.assertTrue(self.client.login(username="u", password="Leeroy"))
 
     def test_confirm_user_password(self):
         self.client.login(username="su", password="pwd")
@@ -901,8 +905,8 @@ class TestModifyUser(TestCase):
             response, "The two password fields didn't match.".replace("'", "&#39;")
             )
         self.client.logout()
-        self.assertEqual(False, self.client.login(username="u", password="Leeroy"))
-        self.assertEqual(True, self.client.login(username="u", password="pwd"))
+        self.assertFalse(self.client.login(username="u", password="Leeroy"))
+        self.assertTrue(self.client.login(username="u", password="pwd"))
 
 class TestAdminFunctions(TestCase):
     def setUp(self):
@@ -1201,9 +1205,15 @@ class TestNotifications(TestCase):
         self.assertEqual(response.status_code, 200)
 
     def test_thread_followers(self):
+        up = UserProfile.objects.get(user=self.u)
         t = Thread.objects.create(
             subject="subject",
-            owner=UserProfile.objects.get(user=self.u),
+            owner=up,
+            )
+        Message.objects.create(
+            thread=t,
+            body="Body",
+            owner=up,
             )
         t.followers = [self.u]
         t.save()
@@ -1220,7 +1230,6 @@ class TestNotifications(TestCase):
         url = reverse("notifications")
         response = self.client.get(url)
         self.assertEqual(response.status_code, 200)
-        self.assertContains(response, "unread")
         self.assertContains(response, "posted")
         self.assertContains(
             response,
