@@ -4,16 +4,17 @@ Project: Farnsworth
 Author: Karandeep Singh Nagra
 '''
 
-from __future__ import unicode_literals
+from __future__ import unicode_literals, absolute_import
 
 from django.contrib.auth.models import User
 from django.conf import settings
-from django.core.urlresolvers import reverse
 from django.db import models
 
 from base.models import UserProfile
+from utils.variables import ANONYMOUS_USERNAME
 from managers.models import Manager
 from workshift.fields import DayField
+from workshift.templatetags.workshift_tags import wurl
 
 WORKSHIFT_MANAGER_VERIFY = "W"
 POOL_MANAGER_VERIFY = "P"
@@ -110,6 +111,9 @@ class Semester(models.Model):
     def is_semester(self):
         return True
 
+    def get_view_url(self):
+        return wurl("workshift:view_semester", sem_url=self.sem_url)
+
 class WorkshiftPool(models.Model):
     title = models.CharField(
         max_length=100,
@@ -193,6 +197,9 @@ class WorkshiftPool(models.Model):
             self.hours, "s" if self.hours != 1 else "",
             )
 
+    def get_view_url(self):
+        return wurl("workshift:view_pool", pk=self.pk, sem_url=self.semester.sem_url)
+
     def is_workshift_pool(self):
         return True
 
@@ -247,6 +254,9 @@ class WorkshiftType(models.Model):
     def is_workshift_type(self):
         return True
 
+    def get_view_url(self):
+        return wurl("workshift:view_type", pk=self.pk)
+
 class TimeBlock(models.Model):
     '''
     A time block to represent member availability during a particular day.
@@ -437,6 +447,13 @@ class WorkshiftProfile(models.Model):
     def get_full_name(self):
         return self.user.get_full_name()
 
+    def get_view_url(self):
+        return wurl(
+            "workshift:profile",
+            targetUsername=self.user.username,
+            sem_url=self.semester.sem_url,
+        )
+
 class RegularWorkshift(models.Model):
     '''
     A weekly workshift for a semester.
@@ -499,6 +516,8 @@ class RegularWorkshift(models.Model):
         )
     addendum = models.TextField(
         default='',
+        blank=True,
+        null=True,
         help_text="Addendum to the description for this workshift.",
         )
 
@@ -514,6 +533,9 @@ class RegularWorkshift(models.Model):
         else:
             return self.workshift_type.title
 
+    def get_view_url(self):
+        return wurl("workshift:view_shift", pk=self.pk, sem_url=self.pool.semester.sem_url)
+
     def display_time(self):
         if self.day is not None:
             ret = self.get_day_display()
@@ -766,8 +788,7 @@ class WorkshiftInstance(models.Model):
         return True
 
     def get_view_url(self):
-        return reverse("workshift:view_instance", kwargs={"pk": self.pk})
-
+        return wurl("workshift:view_instance", pk=self.pk, sem_url=self.semester.sem_url)
 
 def create_workshift_profile(sender, instance, created, **kwargs):
     '''
@@ -775,22 +796,25 @@ def create_workshift_profile(sender, instance, created, **kwargs):
     Parameters:
         instance is an of UserProfile that was just saved.
     '''
-    if instance.status == UserProfile.RESIDENT:
-        try:
-            semester = Semester.objects.get(current=True)
-        except (Semester.DoesNotExist, Semester.MultipleObjectsReturned):
-            pass
-        else:
-            from workshift import utils
-            profile, created = WorkshiftProfile.objects.get_or_create(
-                user=instance.user,
+    if instance.user.username == ANONYMOUS_USERNAME or \
+       instance.status != UserProfile.RESIDENT:
+        return
+
+    try:
+        semester = Semester.objects.get(current=True)
+    except (Semester.DoesNotExist, Semester.MultipleObjectsReturned):
+        pass
+    else:
+        from workshift import utils
+        profile, created = WorkshiftProfile.objects.get_or_create(
+            user=instance.user,
+            semester=semester,
+            )
+        if created:
+            utils.make_workshift_pool_hours(
                 semester=semester,
+                profiles=[profile],
                 )
-            if created:
-                utils.make_workshift_pool_hours(
-                    semester=semester,
-                    profiles=[profile],
-                    )
 
 def create_workshift_pool_hours(sender, instance, **kwargs):
     pool = instance
